@@ -1,12 +1,25 @@
 # -*- coding: utf-8 -*-
 
 from django import forms
+from django.db.models import Q
 from django.contrib import admin
+from django.core.urlresolvers import reverse
+from django.http import HttpResponseRedirect
 from django.utils.translation import ugettext as _
 
 from library import format_minutes_to_time
 import models
 
+# Custom actions
+
+def edit_bookings(modeladmin, request, queryset):
+    ids = [booking.id for booking in queryset]
+    #url = HttpResponseRedirect(reverse('inhouse.admin_views.edit_bookings'))
+    url = "%s?id=" % reverse('inhouse.admin_views.edit_bookings')
+    url += "&id=".join(str(x) for x in ids)
+    return HttpResponseRedirect(url)
+edit_bookings.short_description = _(u'Edit selected bookings')
+
 
 class ModelAdmin(admin.ModelAdmin):
 
@@ -61,7 +74,6 @@ class BillingTypeAdmin(ModelAdmin):
 
     list_display = ('id', 'name', 'description')
     list_display_links = ('id', 'name')
-    ordering = ['id']
 
 
 class BookAdmin(ModelAdmin):
@@ -73,7 +85,8 @@ class BookAdmin(ModelAdmin):
                        'publisher',
                        'edition',
                        'language',
-                       'isbn',
+                       'isbn10',
+                       'isbn13'
                        )}),
         (_(u'Timestamp'), {
             'classes': ('collapse',),
@@ -83,7 +96,7 @@ class BookAdmin(ModelAdmin):
                        'updated_by'
                        )}),
     )
-    list_display = ('id', 'name', 'author', 'publisher', 'isbn',
+    list_display = ('id', 'name', 'author', 'publisher', 'isbn10', 'isbn13',
                     'created', 'updated')
     list_display_links = ('id',)
     list_filter = ['publisher']
@@ -92,6 +105,7 @@ class BookAdmin(ModelAdmin):
 
 class BookingAdmin(ModelAdmin):
 
+    actions = [edit_bookings]
     date_hierarchy = 'created'
     fieldsets = (
         (None, {
@@ -126,15 +140,20 @@ class BookingAdmin(ModelAdmin):
                        'updated_by'
                        )}),
     )
-    list_display = ('id', 'created_by', 'day', 'project', 'step', 'get_title',
-                    'get_duration', 'created', 'updated')
+    list_display = ('id', 'project', 'step', 'get_date', 'get_user',
+                    'get_duration', 'get_tracker', 'get_title')
     list_display_links = ('id', 'get_title')
-    list_filter = ['project', 'created_by']
+    list_filter = ['project', 'day__user', 'issue__tracker', 'invoice_no']
     ordering = ['id']
-    raw_id_fields = ['issue', 'step']
+    raw_id_fields = ['issue', 'step', 'day']
     readonly_fields = ('created', 'created_by', 'updated', 'updated_by')
     search_fields = ['title', 'description', 'issue__title',
-                     'issue__description', 'issue__no']
+                     'issue__description', 'issue__no', 'issue__master']
+
+    def get_date(self, booking):
+        return booking.day.date
+    get_date.short_description = _(u'Date')
+    get_date.admin_order_field = 'day__date'
 
     def get_duration(self, booking):
         return format_minutes_to_time(booking.duration)
@@ -145,6 +164,19 @@ class BookingAdmin(ModelAdmin):
         return booking.get_title()
     get_title.short_description = _(u'Activity')
 
+    def get_tracker(self, booking):
+        if booking.issue:
+            return booking.issue.tracker.name
+        else:
+            return ''
+    get_tracker.short_description = _(u'Tracker')
+    get_tracker.admin_order_field = 'issue__tracker__name'
+
+    def get_user(self, booking):
+        return booking.day.user
+    get_user.short_description = _(u'User')
+    get_user.admin_order_field = 'day__user'
+
 
 class BookingHistoryAdmin(ModelAdmin):
 
@@ -364,6 +396,59 @@ class CustomerAdmin(ModelAdmin):
     readonly_fields = ('created', 'created_by', 'updated', 'updated_by')
 
 
+class DayBookinsInline(admin.TabularInline):
+
+    can_delete = False
+    extra = 0
+    model = models.Booking
+    fields = ('project', 'step', 'duration', 'issue')
+    readonly_fields = ('project', 'step', 'duration', 'issue')
+    template = 'admin/inhouse/booking/day_tabular_inline.html'
+
+
+class DayAdmin(ModelAdmin):
+
+    date_hierarchy = 'date'
+    fieldsets = (
+        (None, {
+            'fields': ('user',
+                       'date',
+                       'locked'
+                       )}),
+        (_(u'Timestamp'), {
+            'classes': ('collapse',),
+            'fields': ('created',
+                       'created_by',
+                       'updated',
+                       'updated_by'
+                       )}),
+    )
+    inlines = [DayBookinsInline]
+    list_display = ('id', 'user', 'date', 'locked', 'get_booking_sum',
+                    'created', 'updated')
+    list_filter = ('user', 'locked')
+    readonly_fields = ('user', 'date', 'created', 'created_by', 'updated',
+                       'updated_by')
+
+    def get_booking_sum(self, day):
+        """Display the booking time per day.
+
+        The normal frontend validation will not allow more than 24 hours,
+        but the backend should display a limit of 24 hours e.g. because of
+        backend jobs that created more than 24 hours per day.
+        """
+        duration = day.get_booking_sum()
+        # TODO: Use constant variable
+        if duration > 1440:
+            color = 'red'
+        else:
+            color = 'black'
+        value = format_minutes_to_time(duration)
+        return '<span style="color: %s;">%s</span>' % (color, value)
+    get_booking_sum.short_description = _(u'Duration')
+    get_booking_sum.allow_tags = True
+
+
 class DepartmentUserInline(admin.TabularInline):
 
     model = models.DepartmentUser
@@ -386,6 +471,7 @@ class DepartmentAdmin(ModelAdmin):
     #inlines = [DepartmentUserInline]
     list_display = ('id', 'name', 'created', 'updated')
     list_display_links = ('id', 'name')
+    ordering = ['name']
     readonly_fields = ('created', 'created_by', 'updated', 'updated_by')
 
 
@@ -408,23 +494,71 @@ class DepartmentUserForm(forms.ModelForm):
 
 class DepartmentUserAdmin(ModelAdmin):
 
+    fieldsets = (
+        (None, {
+            'fields': ('department',
+                       'user',
+                       )}),
+        (_(u'Timestamp'), {
+            'classes': ('collapse',),
+            'fields': ('created',
+                       'created_by',
+                       'updated',
+                       'updated_by'
+                       )}),
+    )
     form = DepartmentUserForm
     list_filter = ['department', 'user']
     list_display = ('id', 'department', 'user')
     list_display_links = ('id',)
+    readonly_fields = ('created', 'created_by', 'updated', 'updated_by')
 
 
 class HardwareAdmin(ModelAdmin):
 
+    fieldsets = (
+        (None, {
+            'fields': ('name',
+                       'manufacturer',
+                       'group',
+                       )}),
+        (_(u'Details'), {
+            'fields': ('model',
+                       'inventory_no',
+                       'serial_no',
+                       )}),
+        (_(u'Timestamp'), {
+            'classes': ('collapse',),
+            'fields': ('created',
+                       'created_by',
+                       'updated',
+                       'updated_by'
+                       )}),
+    )
     list_display = ('id', 'name', 'manufacturer', 'group', 'model',
                     'inventory_no', 'serial_no')
     list_display_links = ('id', 'name')
+    readonly_fields = ('created', 'created_by', 'updated', 'updated_by')
 
 
 class HardwareGroupAdmin(ModelAdmin):
 
+    fieldsets = (
+        (None, {
+            'fields': ('name',
+                       'description',
+                       )}),
+        (_(u'Timestamp'), {
+            'classes': ('collapse',),
+            'fields': ('created',
+                       'created_by',
+                       'updated',
+                       'updated_by'
+                       )}),
+    )
     list_display = ('id', 'name', 'description')
     list_display_links = ('id', 'name')
+    readonly_fields = ('created', 'created_by', 'updated', 'updated_by')
 
 
 class HardwareUserAdmin(ModelAdmin):
@@ -461,6 +595,33 @@ class ManufacturerAdmin(ModelAdmin):
     readonly_fields = ('created', 'created_by', 'updated', 'updated_by')
 
 
+class NewsGroupAdmin(ModelAdmin):
+
+    fieldsets = (
+        (None, {
+            'fields': ('news',
+                       'group',
+                       'priority',
+                       )}),
+        (_(u'Timestamp'), {
+            'classes': ('collapse',),
+            'fields': ('created',
+                       'created_by',
+                       'updated',
+                       'updated_by'
+                       )}),
+    )
+    list_display = ('id', 'news', 'group', 'priority')
+    list_display_links = ('id',)
+    readonly_fields = ('created', 'created_by', 'updated', 'updated_by')
+
+
+class NewsGroupInline(admin.TabularInline):
+
+    model = models.NewsGroup
+    fields = ['group', 'priority']
+
+
 class NewsAdmin(ModelAdmin):
 
     fieldsets = (
@@ -478,6 +639,7 @@ class NewsAdmin(ModelAdmin):
                        'updated_by'
                        )}),
     )
+    inlines = [NewsGroupInline]
     list_display = ('id', 'title', 'valid_from', 'valid_to', 'created',
                     'updated')
     list_display_links = ('id', 'title')
@@ -499,11 +661,14 @@ class ProjectAdmin(ModelAdmin):
     fieldsets = (
         (None, {
             'fields': ('name',
+                       'key',
+                       'image',
                        'description',
                        'type',
                        'status',
                        'master',
                        'department',
+                       'manager',
                        )}),
         (_(u'Customer'), {
             'fields': ('customer',
@@ -512,6 +677,8 @@ class ProjectAdmin(ModelAdmin):
         (_(u'Billing'), {
             'fields': ('billing_type',
                        'commission_status',
+                       'coefficient_saturday',
+                       'coefficient_sunday'
                        )}),
         (_(u'Timestamp'), {
             'classes': ('collapse',),
@@ -522,11 +689,14 @@ class ProjectAdmin(ModelAdmin):
                        )}),
     )
     #form = ProjectForm
-    list_display = ('id', 'name', 'customer', 'type', 'colored_status',
-                    'created', 'updated')
+    list_display = ('id', 'name', 'key', 'customer', 'type', 'colored_status',
+                    'manager', 'created', 'updated')
     list_display_links = ('id', 'name')
-    list_filter = ['status', 'type', 'customer', 'department']
-    ordering = ['id']
+    # TODO: Department filter is not working properly
+    list_filter = ['status', 'type', 'customer']
+    #list_filter = ['status', 'type', 'customer', 'department']
+    list_select_related = True
+    ordering = ['name']
     raw_id_fields = ['customer', 'contact']
     readonly_fields = ('created', 'created_by', 'updated', 'updated_by')
     search_fields = ['name', 'description']
@@ -544,27 +714,104 @@ class ProjectAdmin(ModelAdmin):
     colored_status.admin_order_field = 'status'
     colored_status.short_description = _(u'Status')
 
+    def formfield_for_foreignkey(self, db_field, request, **kwargs):
+        if not request.user.is_superuser:
+            if db_field.name == 'department':
+                query = models.Department.objects.filter(
+                    departmentuser=request.user)
+                #query = query.values_list('id')
+                #kwargs['queryset'] = query
+                #kwargs['to_field_name'] = 'name'
+                return forms.ModelChoiceField(
+                    queryset=query, required=not db_field.blank,
+                    initial=db_field.primary_key)
+                #return db_field.formfield(**kwargs)
+            elif db_field.name == 'master':
+                query = models.Project.objects.filter(
+                    department__departmentuser=request.user)
+                #query = query.values_list('id')
+                #kwargs['queryset'] = query
+                #kwargs['to_field_name'] = 'name'
+                #kwargs['initial'] = db_field.primary_key
+                #kwargs['empty_label'] = db_field.blank and _('None') or None
+                return forms.ModelChoiceField(
+                    queryset=query, required=not db_field.blank,
+                    initial=db_field.primary_key)
+                #return db_field.formfield(**kwargs)
+        return super(ProjectAdmin, self).formfield_for_foreignkey(
+            db_field, request, **kwargs)
+
     def queryset(self, request):
         query = super(ProjectAdmin, self).queryset(request)
         if request.user.is_superuser:
             return query
         # If the user is not an administrator, filter the project list
-        # by his department assignments.
-        departments = models.Department.objects.get_by_user(request.user)
-        ids = [dep.id for dep in departments]
-        return query.filter(department__id__in=ids)
+        # by his department assignments and projects without department
+        # assignment.
+        query = models.Project.objects.filter(
+            Q(department__departmentuser=request.user) |
+            Q(department=None))
+        return query
 
     def save_model(self, request, obj, form, change):
-        # Create a self reference if no master is given
         self_referenced = False
-        if not request.POST.get('master', ''):
+        master = request.POST.get('master')
+        project_manager = request.POST.get('project_manager')
+        if not master:
+            # Create a self reference if no master is given
             self_referenced = True
+        if not change and not project_manager:
+            obj.manager = request.user
         obj.save(user=request.user)
         if self_referenced:
             obj.master = obj
             obj.save(user=request.user)
 
 
+class ProjectRateForm(forms.ModelForm):
+    """Validation form for :class:`ProjectRate`."""
+
+    class Meta:
+        model = models.ProjectRate
+
+    def clean(self, *args, **kwds):
+        data = self.cleaned_data
+        project = data['project']
+        valid_from = data['valid_from']
+        valid_to = data['valid_to']
+        print valid_from, valid_to
+        if valid_from > valid_to:
+            raise forms.ValidationError(_(u'"Valid from" must be smaller than'
+                                          u' "valid to".'))
+        return data
+
+
+class ProjectRateAdmin(ModelAdmin):
+
+    fieldsets = (
+        (None, {
+            'fields': ('project',
+                       'valid_from',
+                       'valid_to',
+                       'hour_rate',
+                       )}),
+        (_(u'Timestamp'), {
+            'classes': ('collapse',),
+            'fields': ('created',
+                       'created_by',
+                       'updated',
+                       'updated_by'
+                       )}),
+    )
+    form = ProjectRateForm
+    list_display = ('id', 'project', 'valid_from', 'valid_to', 'hour_rate',
+                    'created', 'updated')
+    list_display_links = ('id',)
+    list_filter = ('project',)
+    ordering = ('project', '-valid_from')
+    readonly_fields = ('created', 'created_by', 'updated', 'updated_by')
+
+
 class ProjectStatusAdmin(ModelAdmin):
 
     list_display = ('id', 'name', 'description')
@@ -695,15 +942,24 @@ class ProjectUserForm(forms.ModelForm):
 
     def clean(self, *args, **kwds):
         data = self.cleaned_data
+        chosen_project = data['project']
+        if not chosen_project:
+            return data
         # Validates, that user/project is unique
         query = models.ProjectUser.objects.filter(
-            project=data['project'],
+            project=chosen_project,
             user=data['user'])
         if query.count() == 1 and self.instance.pk is None:
             # Raise only if it's a new data set
             raise forms.ValidationError(
-                'The user "%s" is already a member in the project "%s".'
-                % (data['user'], data['project']))
+                _('The user "%s" is already a member in the project "%s".')
+                % (data['user'], chosen_project))
+        chosen_step = data['default_step']
+        if chosen_step and chosen_step.id:
+            if chosen_step.project != chosen_project:
+                raise forms.ValidationError(
+                    _('The chosen step "%s" does not belong to the project "%s".')
+                    % (chosen_step, chosen_project))
         return data
 
 
@@ -805,7 +1061,7 @@ admin.site.register(models.Company, CompanyAdmin)
 admin.site.register(models.Contact, ContactAdmin)
 admin.site.register(models.Country, CountryAdmin)
 admin.site.register(models.Customer, CustomerAdmin)
-# Day
+admin.site.register(models.Day, DayAdmin)
 # DefaultProject
 admin.site.register(models.Department, DepartmentAdmin)
 admin.site.register(models.DepartmentUser, DepartmentUserAdmin)
@@ -815,9 +1071,9 @@ admin.site.register(models.HardwareUser, HardwareUserAdmin)
 admin.site.register(models.Language, LanguageAdmin)
 admin.site.register(models.Manufacturer, ManufacturerAdmin)
 admin.site.register(models.News, NewsAdmin)
-# NewsGroup
+admin.site.register(models.NewsGroup, NewsGroupAdmin)
 admin.site.register(models.Project, ProjectAdmin)
-# ProjectRate
+admin.site.register(models.ProjectRate, ProjectRateAdmin)
 admin.site.register(models.ProjectStatus, ProjectStatusAdmin)
 admin.site.register(models.ProjectStep, ProjectStepAdmin)
 admin.site.register(models.ProjectStepStatus, ProjectStepStatusAdmin)
