@@ -6,19 +6,21 @@ import urllib
 import models
 
 from django.contrib.auth.decorators import login_required
+from django.core.urlresolvers import reverse
 from django.http import (
     HttpResponse,
     HttpResponseNotFound,
     HttpResponseRedirect,
     HttpResponseForbidden)
 
+
 def booking_required(func):
     """Decorator that processes the booking_id handler argument."""
     def booking_wrapper(request, booking_id, *args, **kwds):
         #booking = models.Booking.objects.filter(id=int(booking_id))
         booking = models.Booking.get_by_id(booking_id)
         if not booking:
-            return HttpResponseNotFound('No field exists with that id (%s)' %
+            return HttpResponseNotFound('No field exists with that key (%s)' %
                                         booking_id)
         request.booking = booking
         return func(request, *args, **kwds)
@@ -30,7 +32,7 @@ def booking_owner_required(func):
     @login_required
     @booking_required
     def booking_owner_wrapper(request, *args, **kwds):
-        if request.booking.created_by != request.user:
+        if request.booking.day.user != request.user:
             return HttpResponseForbidden('You do not own this booking')
         return func(request, *args, **kwds)
     return booking_owner_wrapper
@@ -87,7 +89,7 @@ def timer_required(func):
     def timer_wrapper(request, timer_id, *args, **kwds):
         timer = models.Timer.get_by_id(timer_id)
         if not timer:
-            return HttpResponseNotFound('No timer exists with that id (%s)' %
+            return HttpResponseNotFound('No timer exists with that key (%s)' %
                                         timer_id)
         request.timer = timer
         return func(request, *args, **kwds)
@@ -109,4 +111,4 @@ def user_key_required(func):
                                             user_key)
             request.user_to_show = users[0]
         return func(request, *args, **kwds)
-    return user_key_wrapper
\ No newline at end of file
+    return user_key_wrapper
