@@ -1,13 +1,19 @@
 # -*- coding: utf-8 -*-
 
+import calendar
 import cgi
 import datetime
+from decimal import Decimal
+import os
 
+from django.conf import settings
 from django.contrib.auth.models import Group, User
 from django.core.cache import cache
+from django.core.urlresolvers import reverse
 from django.db import models
 from django.utils.translation import ugettext as _
 
+#import filter
 import managers
 from issues.models import Issue, Tracker
 
@@ -15,8 +21,9 @@ from issues.models import Issue, Tracker
 # Roles
 ROLE_ADMINISTRATOR = 1
 ROLE_PROJECT_MANAGER = 2
-ROLE_EMPLOYEE_INTERN = 3
-ROLE_EMPLOYEE_EXTERN = 4
+ROLE_EMPLOYEE_INTERNAL = 3
+ROLE_EMPLOYEE_EXTERNAL = 4
+ROLE_HOLIDAY_MANAGER = 5
 
 # Project status
 PROJECT_STATUS_OPEN = 1
@@ -24,10 +31,23 @@ PROJECT_STATUS_DELETED = 2
 PROJECT_STATUS_IDLE = 3
 PROJECT_STATUS_CLOSED = 4
 
+PROJECT_ACTIVE_STATUS = (
+    PROJECT_STATUS_OPEN,
+)
+
+PROJECT_INACTIVE_STATUS = (
+    PROJECT_STATUS_DELETED,
+    PROJECT_STATUS_IDLE,
+    PROJECT_STATUS_CLOSED,
+)
+
 # Step status
 STEP_STATUS_OPEN = 1
 STEP_STATUS_CLOSED = 2
 
+PROJECT_STEP_INACTIVE_STATUS = (STEP_STATUS_CLOSED,)
+PROJECT_STEP_ACTIVE_STATUS = (STEP_STATUS_OPEN,)
+
 # Address groups
 ADDRESS_GROUP_UNKOWN = 1
 ADDRESS_GROUP_EMPLOYEE = 2
@@ -41,7 +61,7 @@ COUNTRY_UNKNOWN = 1
 # Languages
 LANGUAGE_UNKOWN = 1
 
-
+# Priorities
 PRIORITY_CHOICES = (
     (1, _(u'Low')),
     (2, _(u'Normal')),
@@ -64,6 +84,10 @@ HISTORY_ACTIONS = {HISTORY_ACTION_DELETE: _(u'deleted'),
                    HISTORY_ACTION_ADD: _(u'added'),
                    HISTORY_ACTION_UPDATE: _(u'updated')}
 
+DEFAULT_COEFFICIENT_SATURDAY = Decimal("1.25")
+DEFAULT_COEFFICIENT_SUNDAY = Decimal("1.5")
+DEFAULT_COEFFICIENT_PROJECT_STEP = Decimal("1.0")
+
 
 class Model(models.Model):
     """Abstract model class"""
@@ -76,9 +100,12 @@ class Model(models.Model):
         """Returns a query object by it's primary key.
 
         :param id: Id of the query object
-
-        :returns: Instance of cls
+        :returns: Instance of cls or None
         """
+        try:
+            id = int(id)
+        except (ValueError, TypeError):
+            return None
         query = cls.objects.filter(id=id)
         if query.count() == 0:
             raise cls.DoesNotExist
@@ -115,12 +142,12 @@ class Account(User):
         :param user: A :class:`User` instance
         :returns: :class:`Account`
         """
-        if user.email:
-            query = Account.objects.filter(email=user.email)
-            account = tuple(query)[0] or None
-        else:
-            query = Account.objects.filter(username=user.username)
-            account = tuple(query)[0] or None
+        #if user.email:
+            #query = Account.objects.filter(email=user.email)
+            #account = tuple(query)[0] or None
+        #else:
+        query = Account.objects.filter(username=user.username)
+        account = tuple(query)[0] or None
         if account is not None:
             return account
 
@@ -131,7 +158,7 @@ class Account(User):
         :param cls: :class:`Account`
         :param user: A :class:`User` instance
 
-        :rtype: A string containing the short version of a name
+        :returns: A string containing the short version of a name
         """
         if user.first_name and user.last_name:
             return '%.2s%.3s' % (user.first_name.title(),
@@ -143,7 +170,7 @@ class Account(User):
     def nickname(self):
         """Return ether the first/lastname or the username.
 
-        :returns: Username as string
+        :returns: Username as string or first and last name.
         """
         if self.first_name and self.last_name:
             return u'%s %s' % (self.first_name, self.last_name)
@@ -151,7 +178,7 @@ class Account(User):
             return self.username
 
     def create_profile(self):
-        """Creates an empty profile for a user."""
+        """Creates an empty user profile for an account."""
         user_profile = UserProfile()
         user_profile.user = self
         address = Address()
@@ -165,12 +192,15 @@ class Account(User):
         user_profile.communication = communication
         user_profile.save(user=self)
 
+    # TODO: overload method to create profile automatically?
+    # def get_profile
+
     def get_day(self, date):
         """Returns a :class:`Date` object or creates one.
 
         :param date: datetime.date instance
 
-        :rtype: :class:`Day` instance
+        :returns: :class:`Day` instance
         """
         try:
             day = Day.objects.get(date=date, user=self)
@@ -184,7 +214,7 @@ class Account(User):
         ids = []
         query = Customer.objects.all()
         for customer in query:
-            if not customer.has_bookable_projects():
+            if not customer.has_bookable_projects(user=self):
                 continue
             if not self.works_for_customer(customer):
                 continue
@@ -193,6 +223,10 @@ class Account(User):
         return query
 
     def get_starred_bookings(self):
+        """Return all bookings, that are starred by the user.
+
+        :returns: A :class:`Booking` query.
+        """
         query = Booking.objects.select_related().filter(
             id__in=self.starred_bookings)
         return query
@@ -265,6 +299,11 @@ class Address(Model):
         return self.name1
 
     def get_addressstring(self, sep='<br />', *args, **kwds):
+        """Returns an address as one single string.
+
+        :param sep: Seperator string after earch address element.
+        :returns: String with address information.
+        """
         data = self.get_addresstuple(*args, **kwds)
         if '<' in sep:  # Separator is HTML/XML
             frmt = cgi.escape
@@ -300,6 +339,11 @@ class Address(Model):
         return tuple(t)
 
     def get_join_name(self, join_char=''):
+        """Return only the name fields joined together.
+
+        :param join_char: The character used to join the name fields.
+        :returns: String with address name fields.
+        """
         parts = [self.name1, self.name2,
                  self.name3, self.name4]
         return join_char.join(cgi.escape(part) for part in parts
@@ -362,8 +406,12 @@ class Book(Model):
                                verbose_name=_(u'Edition'))
     language = models.ForeignKey('Language', db_column='bu_sprid',
                                  verbose_name=_(u'Language'))
-    isbn = models.CharField(max_length=14, db_column='bu_isbn', unique=True,
-                            verbose_name=_(u'ISBN'))
+    isbn10 = models.CharField(max_length=10, db_column='bu_isbn10',
+                              blank=True, null=True,
+                                verbose_name=_(u'ISBN-10'))
+    isbn13 = models.CharField(max_length=13, db_column='bu_isbn13',
+                              blank=True, null=True,
+                              verbose_name=_(u'ISBN-13'))
     created_by = models.ForeignKey(User, db_column='bu_crid',
                                    null=True, related_name='book_creators',
                                    verbose_name=_(u'Created by'))
@@ -433,14 +481,20 @@ class Booking(Model):
                              verbose_name=_(u'Project step'))
     issue = models.ForeignKey(Issue, db_column='st_tiid', blank=True,
                               null=True, verbose_name=_(u'Issue'))
-    from_time = models.DecimalField(db_column='st_von',
-                                    max_digits=5, decimal_places=3,
-                                    blank=True, null=True,
-                                    verbose_name=_(u'From'))
-    to_time = models.DecimalField(db_column='st_bis',
-                                  max_digits=5, decimal_places=3,
-                                  blank=True, null=True,
-                                  verbose_name=_(u'To'))
+    from_time = models.TimeField(db_column='st_von',
+                                 blank=True, null=True,
+                                 verbose_name=_(u'Start'))
+    to_time = models.TimeField(db_column='st_bis',
+                               blank=True, null=True,
+                               verbose_name=_(u'End'))
+    #from_time = models.DecimalField(db_column='st_von',
+                                    #max_digits=5, decimal_places=3,
+                                    #blank=True, null=True,
+                                    #verbose_name=_(u'From'))
+    #to_time = models.DecimalField(db_column='st_bis',
+                                  #max_digits=5, decimal_places=3,
+                                  #blank=True, null=True,
+                                  #verbose_name=_(u'To'))
     duration = models.DecimalField(db_column='st_std', max_digits=7,
                                    decimal_places=3,
                                    verbose_name=_(u'Duration'))
@@ -449,13 +503,13 @@ class Booking(Model):
     invoice_no = models.IntegerField(db_column='st_rnr',
                                      blank=True, null=True,
                                      verbose_name=_(u'Invoice no.'))
-    coefficient = models.DecimalField(db_column="st_faktor", max_digits=4,
+    coefficient = models.DecimalField(db_column='st_faktor', max_digits=4,
                                       decimal_places=2, blank=True, null=True,
                                       verbose_name=_(u'External coefficient'))
-    external_coefficient = models.DecimalField(db_column="st_pfaktor",
+    external_coefficient = models.DecimalField(db_column='st_pfaktor',
                                                max_digits=4, decimal_places=2,
                                                blank=True, null=True,
-                                               verbose_name=_(u"Coefficient"))
+                                               verbose_name=_(u'Coefficient'))
     created_by = models.ForeignKey(User, db_column='st_crid',
                                    null=True,
                                    related_name='booking_creators',
@@ -472,13 +526,22 @@ class Booking(Model):
 
     #status = models.ForeignKey('BookingStatus', db_column='st_sts')
 
+    invoice_no.invoice_filter = True
+
     _is_starred = None
 
     class Meta:
         db_table = u'stunde'
+        permissions = (
+            ('can_write', ''),
+            ('can_read', ''),
+            )
         verbose_name = _(u'Booking')
         verbose_name_plural = _(u'Bookings')
 
+    #def __unicode__(self):
+        #return self.get_title()
+
     def copy(self, user):
         copy = Booking()
         for f in self._meta.fields:
@@ -507,13 +570,40 @@ class Booking(Model):
             and self.id in account.starred_bookings
         return self._is_starred
 
-    def get_closing_reason(self):
+    @models.permalink
+    def get_absolute_url(self):
+        #return '/time/edit/%d' % self.id
+        #return reverse('inhouse.views.edit_booking', self.id, [])
+        return ('inhouse.views.edit_booking', [str(self.id)])
+
+    def get_closing_reason_tuple(self):
+        """Return one ore more reasons, why the booking is closed.
+
+        :returns: Tuple with strings
+        """
+        reasons = []
         if not self.is_closed():
-            return None
+            return reasons
         if self.day.locked:
-            return _(u'The day is locked.')
+            reasons.append(_(u'The day is locked.'))
         if self.project.is_closed():
-            return _(u'The project is closed or inactive.')
+            reasons.append(_(u'The project is closed or inactive.'))
+        if self.step.is_closed():
+            reasons.append(_(u'The projectstep is closed.'))
+        return tuple(reasons)
+
+    def get_closing_reason_string(self, sep='<br />'):
+        """Return the closing reason(s) as one single string.
+
+        :returns: String
+        """
+        data = self.get_closing_reason_tuple()
+        if '<' in sep:
+            frmt = cgi.escape
+        else:
+            frmt = lambda x: x
+        return sep.join(frmt(part) for part in data if part is not None)
+
 
     def get_description(self):
         """Returns either the issue description or the issue description.
@@ -526,6 +616,12 @@ class Booking(Model):
             retval = self.description
         return retval or _(u'No description')
 
+    def get_key(self):
+        if self.project:
+            return '%s-%s' % (self.project.key, self.id)
+        else:
+            return str(self.id)
+
     def get_latest_history(self):
         """Get the latest history object.
 
@@ -542,7 +638,7 @@ class Booking(Model):
         :returns: Booking title as string
         """
         if self.issue:
-            retval = u'#%d %s' % (self.issue.no, self.issue.title)
+            retval = self.issue.get_title()
         else:
             retval = self.title
         return retval or _(u'No title')
@@ -565,7 +661,8 @@ class Booking(Model):
 
         :returns: ``True`` or ``False``
         """
-        if self.project.is_closed() or self.day.locked:
+        if (self.project.is_closed() or self.day.locked
+            or self.step.is_closed()):
             return True
         return False
 
@@ -650,14 +747,20 @@ class BookingHistory(Model):
                              verbose_name=_(u'Project step'))
     issue = models.ForeignKey(Issue, db_column='hst_tiid', blank=True,
                               null=True, verbose_name=_(u'Issue'))
-    from_time = models.DecimalField(db_column='hst_von',
-                                    max_digits=5, decimal_places=3,
-                                    blank=True, null=True,
-                                    verbose_name=_(u'From'))
-    to_time = models.DecimalField(db_column='hst_bis',
-                                  max_digits=5, decimal_places=3,
-                                  blank=True, null=True,
-                                  verbose_name=_(u'To'))
+    #from_time = models.DecimalField(db_column='hst_von',
+                                    #max_digits=5, decimal_places=3,
+                                    #blank=True, null=True,
+                                    #verbose_name=_(u'From'))
+    #to_time = models.DecimalField(db_column='hst_bis',
+                                  #max_digits=5, decimal_places=3,
+                                  #blank=True, null=True,
+                                  #verbose_name=_(u'To'))
+    from_time = models.TimeField(db_column='hst_von',
+                                 blank=True, null=True,
+                                 verbose_name=_(u'Start'))
+    to_time = models.TimeField(db_column='hst_bis',
+                               blank=True, null=True,
+                               verbose_name=_(u'End'))
     duration = models.DecimalField(db_column='hst_std', max_digits=7,
                                    decimal_places=3,
                                    verbose_name=_(u'Duration'))
@@ -734,6 +837,10 @@ class BookingHistory(Model):
         return retval or _(u'No description')
 
     def get_title(self):
+        """Returns the title of a booking entry.
+
+        :returns: Issue number and title or booking title.
+        """
         if self.issue:
             retval = u'#%d %s' % (self.issue.no, self.issue.title)
         else:
@@ -813,6 +920,40 @@ class Communication(Model):
         verbose_name = _(u'Communication data')
         verbose_name_plural = _(u'Communication data')
 
+    def __unicode__(self):
+        return self.get_as_string(', ')
+
+    def get_as_string(self, sep='<br />'):
+        """Return the communication data as one single string.
+
+        :param sep: Separator to use.
+        :returns: Single string with data.
+        """
+        data = self.get_tuple()
+        if '<' in sep:
+            frmt = cgi.escape
+        else:
+            frmt = lambda x: x
+        return sep.join(frmt(part) for part in data if part is not None)
+
+    def get_tuple(self):
+        """Return the communication data as tuple elements.
+
+        :returns: Tuple with data
+        """
+        t = []
+        if self.email:
+            t.append(self.email)
+        if self.phone_landline:
+            t.append(self.phone_landline)
+        if self.phone_mobile:
+            t.append(self.phone_mobile)
+        if self.fax:
+            t.append(self.fax)
+        if self.url:
+            t.append(self.url)
+        return tuple(t)
+
 
 class Company(Model):
 
@@ -826,7 +967,7 @@ class Company(Model):
     address = models.ForeignKey('Address', db_column='fir_adrid',
                                 verbose_name=_(u'Address'))
     communication = models.ForeignKey('Communication', db_column='fir_komid',
-                                      verbose_name=_(u'Communcation data'))
+                                      verbose_name=_(u'Communication data'))
     bank = models.CharField(db_column='fir_bbez', max_length=100,
                             blank=True, null=True,
                             verbose_name=_(u'Bank'))
@@ -890,7 +1031,7 @@ class Contact(Model):
                                 blank=True, null=True,
                                 verbose_name=_(u'Address'))
     communication = models.ForeignKey('Communication', db_column='as_komid',
-                                      verbose_name=_(u'Communcation data'))
+                                      verbose_name=_(u'Communication data'))
     birthday = models.DateField(db_column='as_gebdat', blank=True, null=True,
                                 verbose_name=_(u'Birthday'))
     created_by = models.ForeignKey(User, db_column='as_crid',
@@ -925,12 +1066,13 @@ class Country(Model):
     num_code = models.PositiveSmallIntegerField(db_column='la_nr',
                                                 unique=True,
                                                 verbose_name=_(u'Numeric code'))
-    iso2 = models.CharField(db_column='la_iso2', max_length=3, unique=True,
+    iso2 = models.CharField(db_column='la_iso2', max_length=2, unique=True,
                             verbose_name=_(u'CID'))
     iso3 = models.CharField(db_column='la_iso3', max_length=3, unique=True,
                             verbose_name=_(u'CID'))
     dial_code = models.CharField(db_column='la_vorwahl', max_length=10,
-                                 unique=True, verbose_name=_(u'Dial code'))
+                                 null=True,
+                                 verbose_name=_(u'Dial code'))
 
     class Meta:
         db_table = u'k_land'
@@ -960,7 +1102,7 @@ class Customer(Model):
                                 verbose_name=_(u'Address'))
     communication = models.ForeignKey('Communication', db_column='kn_komid',
                                       blank=True, null=True,
-                                      verbose_name=_(u'Communcation data'))
+                                      verbose_name=_(u'Communication data'))
     language = models.ForeignKey('Language', db_column='kn_sprid',
                                  verbose_name=_(u'Language'))
     day_rate = models.DecimalField(db_column='kn_tagessatz', max_digits=14,
@@ -994,8 +1136,13 @@ class Customer(Model):
         return u' '.join([self.name1 or u'', self.name2 or u'',
                           self.name3 or u''])
 
-    def has_bookable_projects(self):
-        query = Project.objects.get_by_customer(self, True)
+    def has_bookable_projects(self, user=None):
+        ids = set()
+        query = Project.objects.get_by_customer(self, True, user)
+        for project in query:
+            if project.get_project_steps(active_only=True).count() > 0:
+                ids.add(project.id)
+        query = Project.objects.filter(id__in=ids)
         return query.count() > 0
 
 
@@ -1046,19 +1193,22 @@ class Day(Model):
         """
         query = Booking.objects.filter(day=self).aggregate(
             models.Sum('duration'))
-        return query['duration__sum']
+        return query['duration__sum'] or 0
 
     def slugify(self):
         """Returns a slugified string of a day.
 
-        :rtype: String
+        :returns: String
         """
-        return self.date.strftime(u'%Y/%m/%d')
+        return self.date.strftime('%Y/%m/%d')
 
 
-class DefaultProject(Model):
+#class DefaultProject(Model):
 
-    pass
+    #id = models.AutoField(db_column='sp_id', primary_key=True)
+    #user = None
+    #tracker = None
+    #project = None
 
 
 class Department(Model):
@@ -1092,14 +1242,29 @@ class DepartmentUser(Model):
 
     id = models.AutoField(db_column='ppg_id', primary_key=True,
                           verbose_name=_(u'Id'),)
-    department = models.ForeignKey('Department', db_column='_abtid')
-    user = models.ForeignKey(User, db_column='_xx')
+    department = models.ForeignKey('Department', db_column='ppg_abtid')
+    user = models.ForeignKey(User, db_column='abt_userid')
+    created_by = models.ForeignKey(User, db_column='ppg_crid',
+                                   null=True,
+                                   related_name='departmentuser_creators',
+                                   verbose_name=_(u'Created by'))
+    created = models.DateTimeField(db_column='ppg_crdate',
+                                   verbose_name=_(u'Created'))
+    updated_by = models.ForeignKey(User, db_column='ppg_updid',
+                                   null=True,
+                                   related_name='departmentuser_updators',
+                                   verbose_name=_(u'Updated by'))
+    updated = models.DateTimeField(db_column='ppg_upddate',
+                                   verbose_name=_(u'Updated'))
 
     class Meta:
         db_table = u'abteilungnutzer'
         verbose_name = _(u'Department member')
         verbose_name_plural = _(u'Department members')
 
+    def __unicode__(self):
+        return u'%s:%s' % (self.department, self.user)
+
 
 class Hardware(Model):
 
@@ -1107,8 +1272,9 @@ class Hardware(Model):
                           verbose_name=_(u'Id'),)
     name = models.CharField(db_column='hw_kbez', max_length=100,
                             verbose_name=_(u'Name'))
-    manufacturer = models.ForeignKey('Manufacturer')
-    group = models.ForeignKey('HardwareGroup')
+    manufacturer = models.ForeignKey('Manufacturer',
+                                     db_column='hw_herid')
+    group = models.ForeignKey('HardwareGroup', db_column='hw_hwgid')
     model = models.CharField(db_column='hw_model', max_length=100,
                              verbose_name=_(u'Model'))
     inventory_no = models.CharField(db_column='hw_inventar',
@@ -1116,6 +1282,18 @@ class Hardware(Model):
                                     verbose_name=_(u'Inventory no.'))
     serial_no = models.CharField(db_column='hw_seriennr',max_length=100,
                                  verbose_name=_(u'Serial no.'))
+    created_by = models.ForeignKey(User, db_column='hw_crid',
+                                   null=True,
+                                   related_name='hardware_creators',
+                                   verbose_name=_(u'Created by'))
+    created = models.DateTimeField(db_column='hw_crdate',
+                                   verbose_name=_(u'Created'))
+    updated_by = models.ForeignKey(User, db_column='hw_updid',
+                                   null=True,
+                                   related_name='hardware_updators',
+                                   verbose_name=_(u'Updated by'))
+    updated = models.DateTimeField(db_column='hw_upddate',
+                                   verbose_name=_(u'Updated'))
 
     class Meta:
         db_table = u'hardware'
@@ -1130,7 +1308,20 @@ class HardwareGroup(Model):
     name = models.CharField(db_column='hwg_kbez', max_length=100,
                             verbose_name=_(u'Name'))
     description = models.CharField(max_length=255, db_column='hwg_bez',
+                                   blank=True, null=True,
                                    verbose_name=_(u'Description'))
+    created_by = models.ForeignKey(User, db_column='hwg_crid',
+                                   null=True,
+                                   related_name='hardwaregroup_creators',
+                                   verbose_name=_(u'Created by'))
+    created = models.DateTimeField(db_column='hwg_crdate',
+                                   verbose_name=_(u'Created'))
+    updated_by = models.ForeignKey(User, db_column='hwg_updid',
+                                   null=True,
+                                   related_name='hardwaregroup_updators',
+                                   verbose_name=_(u'Updated by'))
+    updated = models.DateTimeField(db_column='hwg_upddate',
+                                   verbose_name=_(u'Updated'))
 
     class Meta:
         db_table = u'hardwaregruppe'
@@ -1149,12 +1340,27 @@ class HardwareUser(Model):
                                  verbose_name=_(u'Hardware'))
     user = models.ForeignKey(User, db_column='hwp_userid',
                              verbose_name=_(u'User'))
+    created_by = models.ForeignKey(User, db_column='hwp_crid',
+                                   null=True,
+                                   related_name='hardwareuser_creators',
+                                   verbose_name=_(u'Created by'))
+    created = models.DateTimeField(db_column='hwp_crdate',
+                                   verbose_name=_(u'Created'))
+    updated_by = models.ForeignKey(User, db_column='hwp_updid',
+                                   null=True,
+                                   related_name='hardwareuser_updators',
+                                   verbose_name=_(u'Updated by'))
+    updated = models.DateTimeField(db_column='hwp_upddate',
+                                   verbose_name=_(u'Updated'))
 
     class Meta:
         db_table = u'personhardware'
         verbose_name = _(u'User hardware')
         verbose_name_plural = _(u'User hardware')
 
+    def __unicode__(self):
+        return u'%s:%s' % (self.hardware, self.user)
+
 
 class Language(Model):
 
@@ -1181,9 +1387,11 @@ class Manufacturer(Model):
     name = models.CharField(db_column='her_kbez', max_length=100,
                             unique=True, verbose_name=_(u'Name'))
     address = models.ForeignKey('Address', db_column='her_adrid',
+                                null=True, blank=True,
                                 verbose_name=_(u'Address'))
     communication = models.ForeignKey('Communication', db_column='her_komid',
-                                      verbose_name=_(u'Communcation data'))
+                                      blank=True, null=True,
+                                      verbose_name=_(u'Communication data'))
     created_by = models.ForeignKey(User, db_column='her_crid',
                                    null=True,
                                    related_name='manufacturer_creators',
@@ -1206,6 +1414,33 @@ class Manufacturer(Model):
         return self.name
 
 
+class Message(Model):
+
+    id = models.AutoField(db_column='msg_id', primary_key=True,
+                          verbose_name=_(u'Id'))
+
+    # TODO ...
+
+
+    created_by = models.ForeignKey(User, db_column='msg_crid',
+                                   null=True,
+                                   related_name='message_creators',
+                                   verbose_name=_(u'Created by'))
+    created = models.DateTimeField(db_column='msg_crdate',
+                                   verbose_name=_(u'Created'))
+    updated_by = models.ForeignKey(User, db_column='msg_updid',
+                                   null=True,
+                                   related_name='message_updators',
+                                   verbose_name=_(u'Updated by'))
+    updated = models.DateTimeField(db_column='msg_upddate',
+                                   verbose_name=_(u'Updated'))
+
+    class Meta:
+        db_table = u'nachricht'
+        verbose_name = _(u'Message')
+        verbose_name_plural = _(u'Messages')
+
+
 class News(Model):
 
     id = models.AutoField(db_column='neu_id', primary_key=True)
@@ -1271,6 +1506,11 @@ class Project(Model):
     name = models.CharField(db_column='pr_name', max_length=80,
                             unique=True,
                             verbose_name=_(u'Name'))
+    key = models.CharField(db_column='pr_kurz', max_length=12,
+                           unique=True, verbose_name=_(u'Key'))
+    image = models.ImageField(db_column='pr_bild', blank=True, null=True,
+                              upload_to='projects',
+                              verbose_name=_(u'Image'))
     description = models.TextField(db_column='pr_bez',
                                    blank=True, null=True,
                                    verbose_name=_(u'Description'))
@@ -1289,9 +1529,11 @@ class Project(Model):
     master = models.ForeignKey('Project', db_column='pr_erstprid',
                                blank=True, null=True,
                                verbose_name=_(u'Master project'))
-    department = models.ForeignKey(Department, db_column='pr_abtid',
+    department = models.ForeignKey('Department', db_column='pr_abtid',
                                    blank=True, null=True,
                                    verbose_name=_(u'Department'))
+    manager = models.ForeignKey(User, db_column='pr_leitung', blank=True,
+                                null=True, verbose_name=_(u'Project Manager'))
     billing_type = models.ForeignKey('BillingType', db_column='pr_abaid',
                                      blank=True, null=True,
                                      verbose_name=_(u'Billing type'))
@@ -1302,11 +1544,11 @@ class Project(Model):
     coefficient_saturday = models.DecimalField(
         db_column='pr_faktorsamstag',
         verbose_name=_(u'Coefficient (saturday)'),
-        max_digits=8, decimal_places=2, blank=True, null=True)
+        max_digits=8, decimal_places=2, default=1)
     coefficient_sunday = models.DecimalField(
         db_column='pr_faktorsonntag',
         verbose_name=_(u'Coefficient (sunday)'),
-        max_digits=8, decimal_places=2, blank=True, null=True)
+        max_digits=8, decimal_places=2, default=1)
     created_by = models.ForeignKey(User, db_column='pr_crid',
                                    null=True, related_name='project_creators',
                                    verbose_name=_(u'Created by'))
@@ -1318,6 +1560,9 @@ class Project(Model):
     updated = models.DateTimeField(db_column='pr_upddate',
                                    verbose_name=_(u'Updated'))
 
+    #
+    department.department_filter = True
+
     objects = managers.ProjectManager()
 
     # intnummer
@@ -1331,10 +1576,56 @@ class Project(Model):
     def __unicode__(self):
         return self.name
 
-    def is_closed(self):
-        return self.status.id in (PROJECT_STATUS_CLOSED,
-                                  PROJECT_STATUS_DELETED,
-                                  PROJECT_STATUS_IDLE)
+    @classmethod
+    def copy(cls, other, name=None):
+        new = Project()
+        if name is None:
+            new.name = u'%s \'%s\'' % (_(u'Copy of'), other.name)
+        else:
+            new.name = name
+        new.key = u''
+        new.description = u''
+        new.customer = other.customer
+        new.company = other.company
+        new.contact = other.contact
+        new.type = other.type
+        new.status = other.status
+        new.department = other.department
+        new.manager = other.manager
+        new.billing_type = other.billing_type
+        new.commission_status = other.commission_status
+        new.coefficient_saturday = other.coefficient_saturday
+        new.coefficient_sunday = other.coefficient_sunday
+        return new
+
+    @models.permalink
+    def get_absolute_url(self):
+        return ('inhouse.views.show_project', [str(self.id)])
+
+    def get_coefficient(self, step=None, day=None):
+        """Returns the billing coefficient for each booking.
+
+        :param step:
+        :param day:
+        :returns: Coefficient as integer/float
+        """
+        co_x = 1
+        co_y = 1
+        if day:
+            if day.date.weekday() == calendar.SATURDAY:
+                if self.coefficient_saturday:
+                    co_x = self.coefficient_saturday
+                else:
+                    co_x = DEFAULT_COEFFICIENT_SATURDAY
+            elif day.date.weekday() == calendar.SUNDAY:
+                if self.coefficient_sunday:
+                    co_x = self.coefficient_sunday
+                else:
+                    co_x = DEFAULT_COEFFICIENT_SUNDAY
+        if step and step.coefficient:
+            co_y = step.coefficient
+        return co_x * co_y
+
     def get_description(self):
         """Return the description or an alternate string.
 
@@ -1354,12 +1645,17 @@ class Project(Model):
 
     def get_members(self):
         query = User.objects.filter(projectuser__project=self)
-        #query = query.order_by('')
+        return query
+
+    def get_trackers(self):
+        query = ProjectTracker.objects.filter(project=self)
         return query
 
     def has_trackers(self):
-        query = ProjectTracker.objects.filter(project=self)
-        return query.count() > 0
+        return self.get_trackers().count() > 0
+
+    def is_closed(self):
+        return self.status.id in PROJECT_INACTIVE_STATUS
 
 
 class ProjectRate(Model):
@@ -1371,7 +1667,8 @@ class ProjectRate(Model):
     valid_from = models.DateField(db_column='pps_von',
                                   verbose_name=_(u'Valid from'))
     valid_to = models.DateField(db_column='pps_bis',
-                                verbose_name=_(u'Valid to'))
+                                verbose_name=_(u'Valid to'),
+                                default=datetime.date(4711, 12, 31))
     hour_rate = models.DecimalField(db_column='pps_stdsatz', max_digits=14,
                                    decimal_places=2,
                                    verbose_name=_(u'Hourly rate'))
@@ -1416,11 +1713,11 @@ class ProjectUser(Model):
 
     class Meta:
         db_table = u'projektperson'
-        verbose_name = _(u'Project member')
-        verbose_name_plural = _(u'Project members')
+        verbose_name = _(u'Project user')
+        verbose_name_plural = _(u'Project users')
 
     def __unicode__(self):
-        return '%s: %s' % (self.project, self.user)
+        return u'%s:%s' % (self.project, self.user)
 
 
 class ProjectUserRate(Model):
@@ -1521,6 +1818,8 @@ class ProjectStep(Model):
     updated = models.DateTimeField(db_column='ps_upddate',
                                    verbose_name=_(u'Updated'))
 
+    objects = managers.ProjectStepManager()
+
     # intnummer
 
     class Meta:
@@ -1531,7 +1830,25 @@ class ProjectStep(Model):
     def __unicode__(self):
         return self.name
 
+    @classmethod
+    def copy(cls, other):
+        new = ProjectStep()
+        new.name = other.name
+        new.description = other.description
+        new.status = ProjectStepStatus.get_by_id(STEP_STATUS_OPEN)
+        return new
+
+    def is_closed(self):
+        return self.status.id in PROJECT_STEP_INACTIVE_STATUS
+
+    def is_default(self, user):
+        query = ProjectUser.objects.filter(project=self.project,
+                                           user=user,
+                                           default_step=self.id)
+        return query.count() == 1
+
     def next_position(self):
+        """Set the next free position for a project`s step."""
         steps = ProjectStep.objects.filter(project=self.project)
         # Automatically determine the fields new position
         max = steps.aggregate(models.Max('position'))['position__max'] or 0
@@ -1612,6 +1929,9 @@ class ProjectTracker(Model):
         verbose_name = _(u'Project issue tracker')
         verbose_name_plural = _(u'Project issue tracker')
 
+    def __unicode__(self):
+        return u'%s:%s' % (self.project, self.tracker)
+
 
 class ProjectType(Model):
 
@@ -1754,6 +2074,7 @@ class Timer(Model):
         verbose_name_plural = _(u'Timer')
 
     def clear(self):
+        """Resets the timer."""
         self.status = TIMER_STATUS_STOPPED
         self.duration = 0
 
@@ -1770,6 +2091,10 @@ class Timer(Model):
                 return 0
 
     def get_time_tuple(self):
+        """Returns a split time information in hours and minutes.
+
+        :return: Tuple with hours, minutes
+        """
         minutes = self.duration / 60
         if minutes <= 15:
             # Less then 15 minutes count as 15 minutes
@@ -1786,6 +2111,10 @@ class Timer(Model):
         return h, m
 
     def is_active(self):
+        """Is the timer currently active?
+
+        :returns: ``True`` or ``False``
+        """
         return self.status == TIMER_STATUS_ACTIVE
 
     def start(self, title=None):
@@ -1825,7 +2154,7 @@ class UserProfile(Model):
                                 verbose_name=_(u'Address'))
     communication = models.ForeignKey('Communication', db_column='pe_komid',
                                       blank=True, null=True,
-                                      verbose_name=_(u'Communcation data'))
+                                      verbose_name=_(u'Communication data'))
     birthday = models.DateField(db_column='pe_gebdat', blank=True, null=True,
                                 verbose_name=_(u'Birthday'))
     day_rate = models.DecimalField(db_column='pe_tagkosten', max_digits=14,
