@@ -5,6 +5,7 @@ import logging
 import os
 from time import mktime
 
+from django.conf import settings
 from django.contrib import messages
 from django.contrib.auth.decorators import (
     login_required,
@@ -17,8 +18,7 @@ from django.http import (
     HttpResponse,
     HttpResponseNotFound,
     HttpResponseRedirect,)
-from django.shortcuts import render_to_response
-#from django.template import RequestContext
+from django.shortcuts import get_object_or_404, render_to_response
 import django.template
 from django.utils.translation import ugettext as _
 from django.utils import formats
@@ -35,16 +35,8 @@ import forms
 import models
 
 
-# Add the library module to the django template builtins
-_library_name = __name__.rsplit('.', 1)[0] + '.library'
-if not django.template.libraries.get(_library_name, None):
-    django.template.add_to_builtins(_library_name)
-
-
 DEBUG = 'INHOUSEWEB_DEBUG' in os.environ
 
-counter = 0
-
 
 def respond(request, template, params=None):
     """Helper to render a response, passing standard stuff to the response.
@@ -56,18 +48,18 @@ def respond(request, template, params=None):
 
     :raises:  Whatever render_to_response(template, params) raises.
     """
-    global counter
-    counter += 1
+    #global counter
+    #counter += 1
     if params is None:
         params = {}
     must_choose_nickname = False
     if request.user is not None:
         account = models.Account.current_user_account
         #must_choose_nickname = not account.user_has_selected_nickname()
-    params['request'] = request
-    params['counter'] = counter
-    params['user'] = request.user
-    params['account'] = account
+    #params['request'] = request
+    #params['counter'] = counter
+    #params['user'] = request.user
+    #params['account'] = account
     # FIXME: is_dev und debug korrekt ermitteln
     params['is_dev'] = True
     #params['debug'] = DEBUG
@@ -132,11 +124,13 @@ def booking_popup(request):
 
 
 def index(request):
+    """The main index page."""
     if request.user is None:
-        return HttpResponseRedirect('/signin')
+        return HttpResponseRedirect(settings.LOGIN_URL)
     else:
         return dashboard(request)
 
+
 @login_required
 def company(request):
     """Displays the user`s company information"""
@@ -151,11 +145,13 @@ def company(request):
 @login_required
 def dashboard(request):
     """Show the user`s dashboard."""
-    return respond(request, 'dashboard.html', {})
+    current_news =  models.News.objects.get_current(request.user)
+    return respond(request, 'dashboard.html', {'news': current_news})
 
 
 @permission_required('inhouse.change_userprofile')
 def _edit_profile(request):
+    """Edit the user`s profile."""
     user = request.user
     if request.method != 'POST':
         try:
@@ -163,32 +159,37 @@ def _edit_profile(request):
         except models.UserProfile.DoesNotExist:
             user.account.create_profile()
             user_profile = user.get_profile()
-        form = forms.UserProfileForm(initial={
-            'first_name': user.first_name,
-            'last_name': user.last_name,
-            'email': user.email,
-            'street': user_profile.address.street,
-            'zip_code': user_profile.address.zip_code,
-            'city': user_profile.address.city,
-            'country': user_profile.address.country.id,
-            'birthday': user_profile.birthday,
-            'phone_landline': user_profile.communication.phone_landline,
-            'phone_mobile': user_profile.communication.phone_mobile,
-            'fax': user_profile.communication.fax,
-            'url': user_profile.communication.url,
-            'job': user_profile.job,
-            'personnel_no': user_profile.personnel_no})
+        initial = {'first_name': user.first_name,
+                   'last_name': user.last_name,
+                   'email': user.email,
+                   'street': user_profile.address.street,
+                   'zip_code': user_profile.address.zip_code,
+                   'city': user_profile.address.city,
+                   'country': user_profile.address.country.id,
+                   'birthday': user_profile.birthday,
+                   'phone_landline': user_profile.communication.phone_landline,
+                   'phone_mobile': user_profile.communication.phone_mobile,
+                   'fax': user_profile.communication.fax,
+                   'url': user_profile.communication.url,
+                   'job': user_profile.job,
+                   'personnel_no': user_profile.personnel_no}
+        if user_profile.company is not None:
+            initial.update({'company': user_profile.company.id})
+        form = forms.UserProfileForm(initial=initial)
         form.set_country_choices()
+        form.set_company_choices()
         return respond(request, 'profile.html', {
             'form': form})
     form = forms.UserProfileForm(request.POST)
     form.set_country_choices()
+    form.set_company_choices()
     if form.is_valid():
         user_profile = user.get_profile()
         user.first_name = form.cleaned_data['first_name']
         user.last_name = form.cleaned_data['last_name']
         user.email = form.cleaned_data['email']
         user_profile.job = form.cleaned_data['job']
+        user_profile.company = models.Company.get_by_id(form.cleaned_data['company'])
         address = user_profile.address
         if user.first_name and user.last_name:
             address.name1 = '%s %s' % (user.first_name, user.last_name)
@@ -210,7 +211,8 @@ def _edit_profile(request):
         user.save()
         messages.success(request,
                          _(u'Your profile has been sucessfully updated.'))
-        return HttpResponseRedirect(reverse('inhouse.views.dashboard'))
+        #return HttpResponseRedirect(reverse('inhouse.views.dashboard'))
+        return respond(request, 'profile.html', {'form': form})
     else:
         return respond(request, 'profile.html', {'form': form})
 
@@ -227,7 +229,8 @@ def edit_booking(request):
         form = _save_booking(request, booking.id)
         if form.is_valid():
             messages.add_message(request, messages.INFO,
-                                 _(u'Booking has been updated.'))
+                                 _(u'Booking %s has been updated.')
+                                 % booking.get_key())
             return HttpResponseRedirect('/time/%s' % booking.day.slugify())
     else:
         hours, minutes = booking.split_duration()
@@ -245,7 +248,7 @@ def edit_booking(request):
             initial.update({
                 'tracker': booking.issue.tracker.id,
                 'issue_no': booking.issue.no})
-        form = forms.BookingForm(initial=initial)
+        form = forms.BookingForm(initial=initial, request=request)
         form.set_customer_choices(request.user)
         form.set_project_choices(request.user, booking.project.customer)
         form.set_step_choices(request.user, booking.project)
@@ -270,7 +273,7 @@ def _save_booking(request, booking_id=None):
 
     :param request: Request instance
     """
-    form = forms.BookingForm(request.POST)
+    form = forms.BookingForm(request.POST, request=request)
     form.set_customer_choices(request.user)
     form.set_project_choices(request.user,
                              request.POST.get('customer', None))
@@ -289,9 +292,10 @@ def _save_booking(request, booking_id=None):
         step = models.ProjectStep.get_by_id(form.cleaned_data['step'])
         booking.project = project
         booking.step = step
-        booking.duration = int(form.cleaned_data['minutes']) * 15 + int(form.cleaned_data['hours']) * 60
-        #booking.from_time = form.cleaned_data['from_time']
-        #booking.to_time = form.cleaned_data['to_time']
+        booking.duration = int(form.cleaned_data['minutes']) \
+               * 15 + int(form.cleaned_data['hours']) * 60
+        booking.from_time = form.cleaned_data['from_time']
+        booking.to_time = form.cleaned_data['to_time']
         booking.title = form.cleaned_data['title']
         booking.description = form.cleaned_data['description']
         booking.location = form.cleaned_data['location']
@@ -306,6 +310,10 @@ def _save_booking(request, booking_id=None):
         if issue_no and tracker:
             booking.set_issue(tracker, issue_no)
         booking.next_position()
+        coefficient = project.get_coefficient(step, day)
+        # TODO: Different external coefficient?
+        booking.coefficient = coefficient
+        booking.external_coefficient = coefficient
         booking.save(user=request.user)
     return form
 
@@ -324,6 +332,8 @@ def new_booking(request, year=None, month=None, day=None):
     except ValueError:
         return HttpResponseNotFound()
     day = request.user.account.get_day(date)
+    if day.locked:
+        return HttpResponseNotFound()
     expand_boxes = False
     timer = None
     if request.method == 'POST':
@@ -336,18 +346,19 @@ def new_booking(request, year=None, month=None, day=None):
             messages.add_message(request, messages.INFO,
                                  _(u'Booking has been added.'))
             return HttpResponseRedirect('/time/%s' % day.slugify())
-
-
     else:
         initial = {}
         if 'timer_id' in request.GET:
             timer = models.Timer.get_by_id(request.GET['timer_id'])
+            if timer.created_by != request.user:
+                return HttpResponseNotFound(
+                    'Timer doesn\'t belong to the current user.')
             hours, minutes = timer.get_time_tuple()
             initial.update({'title': timer.title,
                             'hours': hours,
                             'minutes': minutes / 15})
         initial.update({'date': date})
-        form = forms.BookingForm(initial=initial)
+        form = forms.BookingForm(initial=initial, request=request)
         form.set_customer_choices(request.user)
         form.set_project_choices(request.user)
         form.set_step_choices(request.user)
@@ -407,7 +418,14 @@ def _copy_bookings(request):
 
 @login_required
 def show_day(request, year=None, month=None, day=None):
-    if request.method == 'POST':
+    """Display the bookings of a day and provide actions.
+
+    :param year: Year of the date to be displayed
+    :param month: Month of the date to be displayed
+    :param day: Day of the date to be displayed
+    """
+    booking_ids = request.POST.getlist('booking_ids')
+    if request.method == 'POST' and len(booking_ids) > 0:
         if 'delete' in request.POST:
             _delete_bookings(request)
             messages.add_message(request, messages.INFO,
@@ -476,22 +494,28 @@ def show_history(request):
 @login_required
 def show_projects(request):
     """Displays the user's projects."""
-    def results(project, user):
-        data = []
-        bookings = models.Booking.objects.values('day__date').annotate(Sum(
-            'duration'))
-        bookings = bookings.filter(day__user=user, project=project)
-        bookings = bookings.order_by('day__date')
-        for booking in bookings:
-            data.append([mktime(booking.get('day__date').timetuple()) * 1000,
-                         float(booking.get('duration__sum') / 60)])
-        return data
-    projects = []
-    query = models.Project.objects.get_by_user(request.user, active_only=False)
-    query = query.order_by('project__name')
-    for project in query:
-        projects.append((project, results(project, request.user), project.get_members()))
-    return respond(request, 'projects.html', {'projects': projects})
+    form = forms.ProjectRequestForm()
+    form.set_project_choices(request.user)
+    projects = models.Project.objects.get_by_user(request.user,
+                                                  active_only=False)
+    projects = projects.order_by('project__customer')
+    inactive_projects = projects.filter(status__in=models.PROJECT_INACTIVE_STATUS)
+    active_projects = projects.filter(status__in=models.PROJECT_ACTIVE_STATUS)
+    return respond(request, 'projects.html', {
+        'form': form,
+        'active_projects': active_projects,
+        'inactive_projects': inactive_projects})
+
+@login_required
+def show_project(request, project_id):
+    try:
+        project = models.Project.get_by_id(project_id)
+    except models.Project.DoesNotExist:
+        return HttpResponseNotFound('No project found with that key (%s)'
+                                    % project_id)
+    participants = project.get_members().order_by('username')
+    return respond(request, 'project.html', {'project': project,
+                                             'participants': participants})
 
 
 @login_required
@@ -523,7 +547,11 @@ def show_user(request, user_id):
 
     :param user_id: Id of the user to show
     """
-    user = User.objects.filter(id=user_id)[0]
+    user = User.objects.filter(id=user_id)
+    if user.count() == 0:
+        return HttpResponseNotFound('The user cannot be found')
+    else:
+        user = user[0]
     projects = models.Project.objects.get_by_user(
         user, active_only=False).order_by('name')
     departments = models.Department.objects.get_by_user(user).order_by('name')
@@ -534,34 +562,48 @@ def show_user(request, user_id):
 
 
 @login_required
-def show_week(request):
+def show_week(request, year, month, day):
+    """Displays a weekly overview."""
     days = []
     day_data = []
     sum_data = []
-    today = datetime.date.today()
-    week_day = today.weekday()
-    first_day = today - datetime.timedelta(days=week_day)
-    last_day = today + datetime.timedelta(days=7 - (week_day + 1))
+    week_sum = 0
+    try:
+        day = datetime.date(int(year), int(month), int(day))
+    except ValueError:
+        return HttpResponseNotFound()
+    #day = datetime.date.today()
+    week_day = day.weekday()
+    first_day = day - datetime.timedelta(days=week_day)
+    last_day = day + datetime.timedelta(days=7 - (week_day + 1))
     for n in xrange(0, 7):
         date = first_day + datetime.timedelta(days=n)
         days.append(date)
-        day_data.append([])
+        day_data.append([date, []])
         x = request.user.account.get_day(date)
         sum_data.append(x.get_booking_sum())
+        week_sum += x.get_booking_sum()
     bookings = models.Booking.objects.select_related().filter(
         day__user=request.user,
         day__date__gte=first_day,
         day__date__lte=last_day)
     bookings = bookings.order_by('position')
     for booking in bookings:
-        day_data[booking.day.date.weekday()].append(booking)
+        day_data[booking.day.date.weekday()][1].append(booking)
+    next_day = first_day + datetime.timedelta(days=7)
+    previous_day = first_day - datetime.timedelta(days=7)
     return respond(request, 'week.html', {
-        'days': days, 'day_data': day_data,
-        'first_day': first_day, 'last_day': last_day,
-        'sum_data': sum_data})
+        'days': days,
+        'day_data': day_data,
+        'first_day': first_day,
+        'last_day': last_day,
+        'sum_data': sum_data,
+        'week_sum': week_sum,
+        'next_day': next_day,
+        'previous_day': previous_day})
 
 
-@post_required
+#@post_required
 @login_required
 @booking_required
 def star_booking(request):
@@ -590,8 +632,6 @@ def add_timer(request):
     timer.title = 'Timer'
     timer.status = models.TIMER_STATUS_STOPPED
     timer.save(user=request.user)
-    #print request.META['HTTP_REFERER']
-    #return HttpResponseRedirect(request.META['HTTP_REFERER'])
     return HttpResponse('OK', status=200)
 
 
@@ -694,7 +734,7 @@ def undo_booking_change(request):
     return HttpResponseRedirect('/time/%s' % day.slugify())
 
 
-@post_required
+#@post_required
 @login_required
 @booking_required
 def unstar_booking(request):
@@ -732,7 +772,6 @@ def user_popup(request):
 @login_required
 @get_required
 def overview(request):
-    print request.GET
     form = forms.SearchForm(request.GET)
     form.set_customer_choices(request.user)
     form.set_project_choices(request.user, request.GET.get('customer', None))
@@ -740,20 +779,20 @@ def overview(request):
     form.set_tracker_choices(request.user, request.GET.get('project', None))
     query = []
     if 'search' in request.GET:
-        print form.is_valid(), form.errors
         if form.is_valid():
             query = models.Booking.objects.all().select_related()
-            data = form.cleaned_data
-            if 'customer' in data:
-                query = query.filter(project__customer=data['customer'])
-            if 'project' in data:
-                query = query.filter(project=data['project'])
-            if 'step' in data:
-                query = query.filter(step=data['step'])
+            query = query.filter(day__user=request.user)
+            #data = form.cleaned_data
+            #if 'customer' in data:
+                #query = query.filter(project__customer=data['customer'])
+            #if 'project' in data:
+                #query = query.filter(project=data['project'])
+            #if 'step' in data:
+                #query = query.filter(step=data['step'])
             query = query.order_by('-day__date')
-            messages.add_message(request, messages.INFO,
-                                 _(u'%d booking(s) matches found.')
-                                 % query.count())
+            #messages.add_message(request, messages.INFO,
+                                 #_(u'%d booking(s) matches found.')
+                                 #% query.count())
     duration_sum = 0
     # Compute the overall duration time
     # TODO: use aggregation?
@@ -761,3 +800,18 @@ def overview(request):
         duration_sum += booking.duration
     return respond(request, 'overview.html', {
         'form': form, 'bookings': query, 'sum': duration_sum})
+
+@login_required
+@post_required
+def request_projectuser(request):
+    """Requests a project membership."""
+    if request.method == 'POST':
+        form = forms.ProjectRequestForm(request.POST)
+        form.set_project_choices(request.user)
+        if form.is_valid():
+            project = get_object_or_404(models.Project,
+                                        id=form.cleaned_data['project'])
+            if project and project.id:
+                messages.success(request, _(u'Your request for project "%s" has'
+                u' been sent.') % project.name)
+    return HttpResponseRedirect(reverse('inhouse.views.show_projects'))
